﻿/**
 * WCFを使用したサンプルプログラム3: DBをリソースとしてPDFを生成しブラウザでダウンロードさせるサンプルプログラムです。
 * 
 * [本サンプルの実行手順]
 * 
 * <pre>
 * 1. Visual Studio 2008以上で新規プロジェクトを作成する (以下は2008での表示名)
 * 1.1 プロジェクト名は「BizStreamWebApplication」
 * 1.2 「プロジェクトの種類」は「Visual C# / Web」
 * 1.3 「テンプレート」は「ASP.NET Webアプリケーション」
 * 2. 「ソリューションエクスプローラ」にて右クリック後、「サービス参照の追加..」をクリックする
 * 2.1 「アドレス」には「http://wsssrv:8080/axis2/services/bizstream?wsdl」を入力し「移動」をクリックする
 * 2.2 認証を要求されるので、ユーザ名を「bizuser1」、パスワードを「bizuser1」と入力し「OK」をクリックする
 * 2.3 「名前空間」には「BizStreamServiceReference」を入力し「OK」をクリックする
 * 3. 本サンプルを上記プロジェクトに上書きし、ビルド後、実行する
 * </pre>
 * 
 * @author Copyright 2001-2018 BrainSellers.com Corp. All Rights Reserved.
 * @version 1.0.1
 * @since 4.9.0
 */

using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Web;
using BizStreamWebApplication.BizStreamServiceReference;

namespace BizStreamWebApplication
{
    public partial class WSS_WCF_Sample3 : System.Web.UI.Page
    {
        // Webサービスサーバのホスト名
        const string WSS_HOST_NAME = "wsssrv";

        // Webサービスサーバ上のサンプルディレクトリの場所
        const string WSS_BIZSTREAM_SAMPLE_DIR = "\\bs\\sample";

        // Webサービスのエンドポイントプリフィックス
        const string WSS_SERVICE_LOCATION = "http://" + WSS_HOST_NAME + ":8080/axis2/services";

        // BASIC認証のユーザ名
        const string USER_NAME = @"bizuser1";

        // BASIC認証のパスワード
        const string PASSWORD = @"bizuser1";

        // 監査ロギング用のユーザ定義
        const string USER_DEF = "AAA";

        // Webサービスサーバ上の入力レイアウト定義ファイルパス
        const string LAYOUT_FILE = WSS_BIZSTREAM_SAMPLE_DIR + "\\xml\\Sample3-doc.xml";

        const string RESOURCE_NAME = "db2-source";

        // ドキュメントレイアウト内のquery-master名
        const string QUERY_MASTER = "sql-statement";

        // ダウンロードファイル名
        const string DOWNLOAD_FILE_NAME = "WSS_WCF_Sample3.pdf";


        protected void Page_Load(object sender, EventArgs e)
        {
            // データの生成
            byte[] data = generate();

            // HTTPヘッダを設定する
            Response.Clear();
            Response.HeaderEncoding = System.Text.Encoding.GetEncoding("SJIS");
            Response.ContentType = "application/pdf";
            Response.AppendHeader("Content-length", data.Length.ToString());
            Response.AppendHeader("Accept-Ranges", "bytes");
            Response.AppendHeader("Content-Disposition", "attachment; filename=\"" +
                DOWNLOAD_FILE_NAME + "\"");
            Response.Flush();

            // PDFをストリームに書き出す
            Response.BinaryWrite(data);
            HttpContext.Current.ApplicationInstance.CompleteRequest();
        }

        private byte[] generate()
        {
            // クライアントを生成
            bizstreamPortTypeClient client =
                new bizstreamPortTypeClient("BizstreamSOAP11port",
                    WSS_SERVICE_LOCATION + "/bizstream?UserDef=" + USER_DEF);

            // 認証の設定
            BindingElementCollection elements = client.Endpoint.Binding.CreateBindingElements();
            elements.Find<HttpTransportBindingElement>().AuthenticationScheme =
                System.Net.AuthenticationSchemes.Basic;
            client.ClientCredentials.UserName.UserName = USER_NAME;
            client.ClientCredentials.UserName.Password = PASSWORD;

            // MTOMを有効化
            elements.Remove<TextMessageEncodingBindingElement>();
            MtomMessageEncodingBindingElement mtom = new MtomMessageEncodingBindingElement();
            mtom.ReaderQuotas.MaxStringContentLength = 100000;
            elements.Insert(0, mtom);
            elements.Find<HttpTransportBindingElement>().KeepAliveEnabled = false;
            client.Endpoint.Binding = new CustomBinding(elements);

            // リクエストメッセージの作成
            generateDataRequest request = new generateDataRequest();

            // 出力ファイルの種類を設定
            request.dataType = new dataType();
            request.dataType.Item = new pdf();

            // 流し込むデータを設定
            request.resource = new resourceData[1];
            request.resource[0] = makeResourceData();

            // レイアウト定義を設定
            request.layoutData = new layoutDefinition[1];
            request.layoutData[0] = new layoutDefinition();
            request.layoutData[0].uri = LAYOUT_FILE;
            request.layoutData[0].dataReference = RESOURCE_NAME;

            // リクエストを送信
            fileData data = client.generateData(request);

            return data.contentData.Value;
        }

        private resourceData makeResourceData()
        {
            resourceData resData = new resourceData();
            resData.name = RESOURCE_NAME;

            resData.queryParameter = new queryParameter[1];
            resData.queryParameter[0] = new queryParameter();
            // ドキュメントレイアウト内のリソース名を指定
            resData.queryParameter[0].masterName = QUERY_MASTER; 

            return resData;
        }
    }
}
